@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2004 Stephen Bach
+	Copyright (C) 2004, 2005 Stephen Bach
 	This file is part of the viewglob package.
 
 	viewglob is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 #include "feedback.h"
 #include "file_box.h"
 #include "wrap_box.h"
+#include "lscolors.h"
 #include <gtk/gtk.h>
 #include <string.h>      /* For strcmp */
 
@@ -65,7 +66,7 @@ static gint cmp_same_name(gconstpointer a, gconstpointer b);
 static gint cmp_ordering_ls(gconstpointer a, gconstpointer b);
 static gint cmp_ordering_win(gconstpointer a, gconstpointer b);
 
-static void initialize_icons(void);
+static void initialize_icons(gchar* test_string);
 static GdkPixbuf*  make_pixbuf_scaled(const guint8 icon_inline[], gint scale_height);
 
 
@@ -78,10 +79,6 @@ static GdkPixbuf* file_type_icons[FILE_TYPE_ICONS_COUNT] =
 
 static GCompareFunc ordering_func = cmp_ordering_ls;
 
-static gchar* fitem_font_tags_open = NULL;
-static gchar* fitem_font_tags_close = NULL;
-
-
 /* --- functions --- */
 GType file_box_get_type(void) {
 	static GType file_box_type = 0;
@@ -340,66 +337,50 @@ void file_box_set_icon(FileType type, GdkPixbuf* icon) {
 	file_type_icons[type] = icon;
 }
 
-/* Set the size of FItem font (and the size of the file type icons). */
+/* Set the size of the FItem font (and the size of the file type icons). */
 void file_box_set_sizing(gint modifier) {
 
-	gint size;
-	gchar* temp;
-
-	gint i;
-
 	g_return_if_fail(modifier <= 10);   /* Constrain between -10 and 10. */
 	g_return_if_fail(modifier >= -10);
 
-	g_free(fitem_font_tags_open);
-	g_free(fitem_font_tags_close);
+	gint i;
+	GString* test_string = g_string_new("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ123456789");
 
-	fitem_font_tags_open = g_strdup("");
-	fitem_font_tags_close = g_strdup("");
+	gint font_size_modifier = BASE_FONT_SIZE + modifier;
 
-	size = BASE_FONT_SIZE + modifier;
-
-	/* Create the tag strings. */
-	if (size > 0) {
-		for (i = 0; i < size; i++) {
-			temp = fitem_font_tags_open;
-			fitem_font_tags_open = g_strconcat(fitem_font_tags_open, "<big>", NULL);
-			g_free(temp);
-
-			temp = fitem_font_tags_close;
-			fitem_font_tags_close = g_strconcat(fitem_font_tags_close, "</big>", NULL);
-			g_free(temp);
+	/* Create a test string. */
+	if (font_size_modifier > 0) {
+		for (i = 0; i < font_size_modifier; i++) {
+			g_string_prepend(test_string, "<big>");
+			g_string_append(test_string, "</big>");
 		}
 	}
-	else if (size < 0) {
-		for (i = 0; i > size; i--) {
-			temp = fitem_font_tags_open;
-			fitem_font_tags_open = g_strconcat(fitem_font_tags_open, "<small>", NULL);
-			g_free(temp);
-
-			temp = fitem_font_tags_close;
-			fitem_font_tags_close = g_strconcat(fitem_font_tags_close, "</small>", NULL);
-			g_free(temp);
+	else if (font_size_modifier < 0) {
+		for (i = font_size_modifier; i < 0; i++) {
+			g_string_prepend(test_string, "<small>");
+			g_string_append(test_string, "</small>");
 		}
 	}
 
 	/* (Re)set the icons with the new sizing. */
-	initialize_icons();
+	initialize_icons(test_string->str);
+
+	/* Parse LS_COLORS. */
+	parse_ls_colors(font_size_modifier);
+
+	g_string_free(test_string, TRUE);
 }
 
 
-static void initialize_icons(void) {
+static void initialize_icons(gchar* test_string) {
 #include "file_icons.h"
 
 	GtkIconTheme* current_theme;
 	GtkIconInfo* icon_info;
 
-	const gchar* test_string = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ123456789";
-
 	guint icon_size;
 	GtkWidget* test_label;
 	PangoLayout* test_layout;
-	gchar* label_markup;
 
 	int i;
 
@@ -412,15 +393,10 @@ static void initialize_icons(void) {
 	}
 
 	/* Figure out a good size for the file type icons. */
-	if (fitem_font_tags_open && fitem_font_tags_close)
-		label_markup = g_strconcat(fitem_font_tags_open, test_string, fitem_font_tags_close, NULL);
-	else
-		label_markup = g_strdup(test_string);
 	test_label = gtk_label_new(NULL);
-	gtk_label_set_markup(GTK_LABEL(test_label), label_markup);
+	gtk_label_set_markup(GTK_LABEL(test_label), test_string);
 	test_layout = gtk_label_get_layout(GTK_LABEL(test_label));
 	pango_layout_get_pixel_size(test_layout, NULL, &icon_size);
-	g_free(label_markup);
 	gtk_widget_destroy(test_label);
 
 	/* Try to get icons from the current theme. */
@@ -706,7 +682,7 @@ void file_box_flush(FileBox* fbox) {
 static void file_box_size_request(GtkWidget* widget, GtkRequisition* requisition) {
 	FileBox* this = FILE_BOX(widget);
 	if (!this->eat_size_requests) {
-		//g_printerr(".");
+		/*g_printerr(".");*/
 		wrap_box_size_request_optimal(widget, requisition, this->optimal_width);
 	}
 }
@@ -734,8 +710,7 @@ static void fitem_build_widgets(FItem* fi) {
 	GtkWidget* hbox;
 	GtkWidget* eventbox;
 
-	gchar* label_markup;
-	gchar* temp;
+	gchar* label_text;
 	gsize  length;
 
 	/* Event Box (to show selection) */
@@ -757,22 +732,13 @@ static void fitem_build_widgets(FItem* fi) {
 		gtk_box_pack_start(GTK_BOX(hbox), icon_image, FALSE, FALSE, 0);
 	}
 
-	/* Label -- must convert the text to utf8, then escape markup special characters. */
-	label_markup = g_filename_to_utf8(fi->name, strlen(fi->name), NULL, &length, NULL);
-	temp = g_markup_escape_text(label_markup, length);
-	g_free(label_markup);
+	/* Label -- must convert the text to utf8. */
+	label_text = g_filename_to_utf8(fi->name, strlen(fi->name), NULL, &length, NULL);
+	label = gtk_label_new(label_text);
+	g_free(label_text);
 
-	/* Add beginning and end tags, if present. */
-	if (fitem_font_tags_open && fitem_font_tags_close) {
-		label_markup = g_strconcat(fitem_font_tags_open, temp, fitem_font_tags_close, NULL);
-		g_free(temp);
-	}
-	else
-		label_markup = temp;
-
-	label = gtk_label_new(NULL);
-	gtk_label_set_markup(GTK_LABEL(label), label_markup);
-	g_free(label_markup);
+	/* LS_COLORS. */
+	label_set_attributes(fi->name, fi->type, GTK_LABEL(label));
 
 	gtk_misc_set_padding(GTK_MISC(label), 1, 0);
 	gtk_widget_show(label);
