@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2004 Stephen Bach
+	Copyright (C) 2004, 2005 Stephen Bach
 	This file is part of the viewglob package.
 
 	viewglob is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <gdk/gdkx.h>
 #include <string.h>       /* For strcmp. */
 #include <unistd.h>       /* For getopt. */
 #include <stdio.h>        /* For BUFSIZ. */
@@ -36,6 +37,7 @@
 /* Prototypes. */
 static gboolean  receive_data(GIOChannel* source, gchar* buff, gsize size, gsize* bytes_read);
 static GString*  read_string(const gchar* buff, gsize* start, gsize n, gchar delim, struct holdover* ho, gboolean* finished);
+static void      write_xwindow_id(GtkWidget* gtk_window);
 
 static void        set_icons(void);
 
@@ -240,6 +242,7 @@ static void process_cmd_data(const gchar* buff, gsize bytes, Exhibit* e) {
 	GString* string = NULL;
 	gboolean completed = FALSE;
 
+	gchar* cmdline_utf8;
 	gsize pos = 0;
 
 	#if DEBUG_ON
@@ -286,9 +289,11 @@ static void process_cmd_data(const gchar* buff, gsize bytes, Exhibit* e) {
 			case CRS_CMD:
 				string = read_string(buff, &pos, bytes, '\n', &ho, &completed);
 				if (completed) {
-					/* Set the cmdline text. */
+					/* Set the cmdline text after converting to UTF8. */
 					DEBUG((df, "cmd: %s\n", string->str));
-					gtk_entry_set_text(GTK_ENTRY(e->cmdline), string->str);
+					cmdline_utf8 = g_filename_to_utf8(string->str, string->len, NULL, NULL, NULL);
+					gtk_entry_set_text(GTK_ENTRY(e->cmdline), cmdline_utf8);
+					g_free(cmdline_utf8);
 					rs = CRS_DONE;
 					advance = TRUE;
 					g_string_free(string, TRUE);
@@ -395,7 +400,7 @@ static void process_glob_data(const gchar* buff, gsize bytes, Exhibit* e) {
 
 					if (dir_rank == 1) {
 						/* Put pwd into the window title. */
-						char* title = g_strconcat("gviewglob - ", string->str, NULL);
+						char* title = g_strconcat("gviewglob ", string->str, NULL);
 						gtk_window_set_title(GTK_WINDOW(e->window), title);
 						g_free(title);
 					}
@@ -568,7 +573,6 @@ static gboolean window_key_press_event(GtkWidget* window, GdkEventKey* event, gp
 			break;
 
 		default:
-
 			/* The rest are passed to the terminal. */
 			temp1 = g_malloc(2);
 			*temp1 = event->keyval;
@@ -607,6 +611,25 @@ static gboolean window_key_press_event(GtkWidget* window, GdkEventKey* event, gp
 }
 
 
+/* Send the id of the display's X window to seer. */
+static void write_xwindow_id(GtkWidget* gtk_window) {
+
+	GdkWindow* gdk_window = gtk_window->window;
+
+	if (gdk_window) {
+		GString* xwindow_string = g_string_new(NULL);
+
+		g_string_printf(xwindow_string, "xid:%lu", GDK_WINDOW_XID(gdk_window));
+		feedback_write_string(xwindow_string->str, xwindow_string->len);
+
+		g_string_free(xwindow_string, TRUE);
+	}
+	else
+		g_warning("Couldn't find an id for the display's window.\n"
+		          "Therefore the focus shortcut won't work :-(");
+}
+
+
 static gboolean parse_args(int argc, char** argv) {
 	gboolean in_loop = TRUE;
 	gint max;
@@ -810,6 +833,9 @@ int main(int argc, char *argv[]) {
 	/* And we're off... */
 	gtk_widget_show(e.window);
 
+	/* Pass the window ID back to seer. */
+	write_xwindow_id(e.window);
+
 	gtk_main();
 
 #if DEBUG_ON
